/*-----------------------------------------------------------------------------

Copyright (C) 1990, 1993, 1994, 2002, 2003, 2006.

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659
USA   

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

-------------------------------------------------------------------------------

Function      starbox  - puts title in a centered box of stars.

Syntax        #include "sclfuncs.h"
              char* starbox(const char *title, const char terminator);

Prototype in  sclfuncs.h

Description   title is a character string.  The string is made up of  
              concatonated lines.  A line is made up of 0-68 characters 
              followed by a slash (default).  Examples are:
              cout << starbox("//line 1/line 2/line 3///");
              cout << starbox("\n\nline 1\nline 2\nline 3\n\n\n",'\n');

Return value  Character string.

Functions     Library: strcat, strchr, strlen
called        Libcpp:   (none)

-----------------------------------------------------------------------------*/

#include "sclfuncs.h"
using std::strcat;
using std::strchr;
using std::strlen;

const char* scl::starbox(const char *title, const char terminator)
{
  const char *begin;
  char       border[81],tab[81],work[256],middle[81];
  char       *line, *t;
  const char *end;
  int        i,pad,mpad,linesize;
  int        length;
  long int   cum_length;

  static char starbox_str_pointer[1024];
  long int    starbox_str_length = 1024;

  cum_length = 0;
  *starbox_str_pointer = '\0';

  for (i=0; i<70; i++)
    border[i] = '*';
  border[70] = '\n'; border[71] = '\0';

  ((LINESIZE<72)||(LINESIZE>133)) ? (linesize=133) : (linesize=LINESIZE);
  
  pad = (linesize-72)/2+1;
  for (i=0; i<pad; i++)
    tab[i] = ' ';
  tab[pad] = '\0';

  work[0]='\n'; work[1]='\n'; work[2]='\0';
  line = strcat(work,tab);
  line = strcat(work,border);

  cum_length += strlen(line);

  if (cum_length < starbox_str_length)
    strcat(starbox_str_pointer,line);
  else 
    return "Error, starbox, title too long.";

  begin = title;
  while ( (end=strchr(begin,terminator)) != 0 ) {
    length = end - begin;
    if (length > 68) return "Error, starbox, line too long.";
    middle[0] = '*';
    mpad = (68-length)/2+1;
    for (i=1; i<=mpad; i++) middle[i] = ' ';
    t = &middle[mpad];
    while (begin < end) *t++ = *begin++;
    begin++;
    for (i=mpad+length; i<=68; i++) middle[i]=' ';
    middle[69]='*'; middle[70]='\n'; middle[71]='\0';
    work[0]='\0';
    line = strcat(work,tab);
    line = strcat(work,middle);


    if (cum_length < starbox_str_length)
      strcat(starbox_str_pointer,line);
    else 
      return "Error, starbox, title too long.";

  }

  work[0]='\0';
  line = strcat(work,tab);
  line = strcat(work,border);

  if (cum_length < starbox_str_length)
    strcat(starbox_str_pointer,line);
  else 
    return "Error, starbox, title too long.";

  return starbox_str_pointer;

}



