/*-----------------------------------------------------------------------------

Copyright (C) 2004, 2006.

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659
USA   

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

------------------------------------------------------------------------------

Function      drinv - Inverts an upper triangular matrix in place 

Syntax        #include "libsnp.h"
              void libsnp::drinv(REAL* a, INTEGER n)

Prototype in  libsnp.h

Description   a is an (n*n+n)/2-vector containing the upper triangle of
              an upper triangular matrix stored columnwize which
              contains the matrix to be inverted on entry and the
              inverse, which is upper triangular on return.

Remark        drinv.cpp is drinv.f translated to C++.

Reference     None.

Return value  None.

Functions     Library: (none)
called        Libcpp:  (none)

-----------------------------------------------------------------------------*/

#include "libsnp.h"

using namespace scl;
using namespace std;

using namespace libsnp;

void libsnp::drinv(REAL* a, INTEGER n)
{
  REAL din,work;
  INTEGER ipiv,ind,min,kend,lanf,lhor,lver;
  INTEGER i,j,k,l;
  ipiv = (n*n+n)/2;
  ind = ipiv;
  for (i=1; i<=n; ++i) {
    din = 1.0/a[ipiv-1];
    a[ipiv-1] = din;
    min = n;
    kend = i - 1;
    lanf = n - kend;
    if (kend>0) {
      j=ind;
      for (k=1; k<=kend; ++k) {
        work = 0.0;
        --min;
        lhor = ipiv;
        lver = j;
        for (l=lanf; l<=min; ++l) {
          ++lver;
          lhor += l;
          work += a[lver-1]*a[lhor-1];
        }
        a[j-1] = -work*din;
        j -= min;
      }
    } 
    ipiv -= min;
    --ind;
  }
}


