/*-----------------------------------------------------------------------------

Copyright (C) 2004, 2006.

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659
USA   

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

------------------------------------------------------------------------------

Function      gen2sym - converts n by n symmetic matrix stored as a
                        general matrix to compressed storaage.  All
                        storage is columnwise.

Syntax        #include "libsnp.h"
              void libsnp::gen2sym(REAL* a, INTEGER n)

Prototype in  libsnp.h

Description   on entry a is an n*n vector containing the columns of a.
              on return a is an (n*n+n)/2-vector containing the upper 
              triangle of a stored columnwize 

Remark        None.

Reference     None.

Return value  None.

Functions     Library: (none)
called        Libcpp:  (none)

-----------------------------------------------------------------------------*/

#include "libsnp.h"

using namespace scl;
using namespace std;

using namespace libsnp;

void libsnp::gen2sym(REAL* a, INTEGER n)
{
  for (INTEGER j=1; j<=n; ++j) {
    for (INTEGER i=1; i<=j; ++i) {
      a[(j*j-j)/2 + i - 1] = a[n*(j-1) + i - 1];
    }
  }
}


/*-----------------------------------------------------------------------------

Copyright (C) 2004, 2006.

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659
USA   

Permission to use, copy, modify, and distribute this software and its docu-
mentation for any purpose and without fee is hereby granted, provided that 
the above copyright notice appears in all copies and that both that copyright 
notice and this permission notice appear in supporting documentation.  This 
software is provided "as is" without any expressed or implied warranty.

------------------------------------------------------------------------------

Function      gen2upr - converts n by n upper triangular matrix stored as 
                        a general matrix to compressed storaage.  All
                        storage is columnwise.

Syntax        #include "libsnp.h"
              void libsnp::gen2upr(REAL* a, INTEGER n)

Prototype in  libsnp.h

Description   on entry a is an n*n vector containing the columns of a.
              on return a is an (n*n+n)/2-vector containing the upper 
              triangle of a stored columnwize 

Remark        None.

Reference     None.

Return value  None.

Functions     Library: (none)
called        Libcpp:  (none)

-----------------------------------------------------------------------------*/

#include "libsnp.h"

using namespace scl;
using namespace std;

using namespace libsnp;

void libsnp::gen2upr(REAL* a, INTEGER n)
{
  for (INTEGER j=1; j<=n; ++j) {
    for (INTEGER i=1; i<=j; ++i) {
      a[(j*j-j)/2 + i - 1] = a[n*(j-1) + i - 1];
    }
  }
}


/*-----------------------------------------------------------------------------

Copyright (C) 2004, 2006.

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659
USA   

Permission to use, copy, modify, and distribute this software and its docu-
mentation for any purpose and without fee is hereby granted, provided that 
the above copyright notice appears in all copies and that both that copyright 
notice and this permission notice appear in supporting documentation.  This 
software is provided "as is" without any expressed or implied warranty.

------------------------------------------------------------------------------

Function      sym2gen - converts n by n symmetic matrix stored as a
                        compressed  matrix to general storaage.  All
                        storage is columnwise.

Syntax        #include "libsnp.h"
              void libsnp::sym2gen(REAL* a, INTEGER n)

Prototype in  libsnp.h

Description   on entry a is an (n*n+n)/2-vector containing the upper 
              triangle of a stored columnwise.  on return a is an n*n 
              vector containing the columns of a

Remark        None.

Reference     None.

Return value  None.

Functions     Library: (none)
called        Libcpp:  (none)

-----------------------------------------------------------------------------*/

#include "libsnp.h"

using namespace scl;
using namespace std;

using namespace libsnp;

void libsnp::sym2gen(REAL* a, INTEGER n)
{
  for (INTEGER j=n; j>=1; --j) {
    for (INTEGER i=j; i>=1; --i) {
      a[n*(j-1) + i - 1] = a[(j*j-j)/2 + i - 1];
    }
  }
  for (INTEGER j=1; j<=n; ++j) {
    for (INTEGER i=1; i<=j; ++i) {
      a[n*(i-1) + j - 1] = a[n*(j-1) + i - 1];
    }
  }
}


/*-----------------------------------------------------------------------------

Copyright (C) 2004, 2006.

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659
USA   

Permission to use, copy, modify, and distribute this software and its docu-
mentation for any purpose and without fee is hereby granted, provided that 
the above copyright notice appears in all copies and that both that copyright 
notice and this permission notice appear in supporting documentation.  This 
software is provided "as is" without any expressed or implied warranty.

------------------------------------------------------------------------------

Function      upr2gen - converts n by n upper triangular matrix stored
                        as a compressed  matrix to general storaage.  All
                        storage is columnwise.

Syntax        #include "libsnp.h"
              void libsnp::upr2gen(REAL* a, INTEGER n)

Prototype in  libsnp.h

Description   on entry a is an (n*n+n)/2-vector containing the upper 
              triangle of a stored columnwise.  on return a is an n*n 
              vector containing the columns of a

Remark        None.

Reference     None.

Return value  None.

Functions     Library: (none)
called        Libcpp:  (none)

-----------------------------------------------------------------------------*/

#include "libsnp.h"

using namespace scl;
using namespace std;

using namespace libsnp;

void libsnp::upr2gen(REAL* a, INTEGER n)
{
  for (INTEGER j=n; j>=1; --j) {
    for (INTEGER i=j; i>=1; --i) {
      a[n*(j-1) + i - 1] = a[(j*j-j)/2 + i - 1];
    }
  }
  for (INTEGER j=1; j<=n; ++j) {
    for (INTEGER i=1; i<j; ++i) {
      a[n*(i-1) + j - 1] = 0.0;
    }
  }
}

