#include <sstream>
#include <cerrno>
#include "libscl.h"
#include "sim.h"
#include "transf.h"
using namespace scl;
using namespace std;


int main()
  {
// THE SEED

  INT_32BIT seed = 19801018;
  INTEGER start = 1;
  INTEGER nfiles = 1000;

//THE PARAMETERS

//THE PARAMETERS--COMPOUND POISSON WITH UNIFORM JUMPS

  REAL kappa_1 = 0.02; REAL theta = 0.5; REAL sigma = 0.07; 
  REAL kappa_2 = 0.50; REAL lambda_v = 1.0; REAL mu_v = 4.0;
  REAL lambda =  1.00; REAL mu_p =  0.0; REAL sigma_p =sqrt(0.2)/sqrt(1.0); 
  REAL ind_p = 1.0; REAL ind_v = 1.0;

//ind_p is for the price jumps, ind_v for the dependent vol jumps

  realmat rho(14,1,0.0);
  rho[1] = kappa_1; rho[2] = theta; rho[3] = sigma; rho[4] = kappa_2; rho[5] = ind_p; rho[6] = ind_v;
  rho[7] = lambda; rho[8] = mu_p; rho[9] = sigma_p; rho[10] = lambda_v; rho[11] = mu_v;

//CONTROL VARIABLES

  INTEGER T = 4000; 
  INTEGER freq = 400; //#of hf obs. per day
  INTEGER neuler = 3; //number of euler discretizations
  INTEGER T_limit = 100000; //number of days for the true LT
  INTEGER K = 10; //# of lags for the JLT
  INTEGER N = 4; 
  vector<REAL> uvec(N,0.0); uvec[0] = 0.0; uvec[1] = 0.50; uvec[2] = 1.25; uvec[3] = 2.50;

  REAL beta = 2.0;

/////////////////////////////////////////////

  string filename_a; string filename_b; string temp_a = "spot_"; string temp_b = "true_"; string temp_case = "G_";
  char lag[5]; sprintf(lag,"%d",K); string temp2 = ".dat"; filename_a = temp_a+temp_case+lag+temp2; filename_b = temp_b+temp_case+lag+temp2; 

  realmat vols = vol(seed,T_limit, neuler*freq, rho);
  realmat RLT_true = LT(vols,T_limit,neuler*freq,uvec); realmat out_temp_true = JLT(RLT_true,K); realmat out_true(3,pow(N,2.0),0.0);
  for (INTEGER j=1; j<=N; j++) {
    for (INTEGER l=1; l<=N; l++) {
      out_true(1,(l-1)*N+j) = uvec[j-1]; out_true(2,(l-1)*N+j) = uvec[l-1];//first element is current day
      out_true(3,(l-1)*N+j) = out_temp_true(j,l);
    }
  }
  
  vecwrite(filename_b.c_str(),out_true); 
  //vecwrite("vol.dat",vols);

  realmat out_spot(nfiles+2,pow(N,2.0),0.0);
  for (INTEGER i=1; i<=N; i++) {
    for (INTEGER j=1; j<=N; j++) {
      out_spot(1,(i-1)*N+j) = uvec[j-1]; out_spot(2,(i-1)*N+j) = uvec[i-1];//first element is current day 
    }
  }
  for (INTEGER i=start; i<=nfiles; i++) {
     char number[5]; sprintf(number,"%d",i);
     string filename_ret = "ret_"+temp_case+"_"+number+temp2;
     realmat rets = sim(seed, T, freq, neuler, rho);
     realmat RLT_spot = RLT(rets, beta, T, freq, uvec);
     realmat out_temp_spot = JLT(RLT_spot,K);
     cout<<i<<'\n';
     for (INTEGER j=1; j<=N; j++) {
        for (INTEGER l=1; l<=N; l++) {
          out_spot(i+2,(l-1)*N+j) = out_temp_spot(j,l);
        }
     }
//     vecwrite(filename_ret.c_str(),rets);
  }  
  vecwrite(filename_a.c_str(),out_spot); 

  return 0;
  }

