#include <sstream>
#include <cerrno>
#include "libscl.h"
#include "libsnp.h"


using namespace scl;
using namespace std;

realmat jumps(INT_32BIT& seed, REAL& T, realmat& pars)
{
  REAL lambda = pars[1]; REAL mu_p = pars[2]; REAL sigma_p = pars[3]; REAL mu_v = pars[4];

  realmat output(2,1,0.0);
  REAL gam =-log(ran(seed));
  while(gam<lambda*T){
    gam += - log(ran(seed));
    REAL shot_p = mu_p+unsk(seed)*sigma_p;
    output[1]+= shot_p;
    REAL shot_v = -log(ran(seed))/mu_v;
    output[2]+= shot_v;
  }
  return output;
}

realmat sim(INT_32BIT& seed, INTEGER& T, INTEGER& freq, INTEGER& neuler, realmat& rho)
{
  realmat output(T*freq,1,0.0);
  REAL kappa_1 = rho[1];
  REAL theta = rho[2];
  REAL sigma = rho[3];
  REAL kappa_2 = rho[4];
  REAL ind_p = rho[5];
  REAL ind_v = rho[6];
  REAL delta = 1.0/REAL(freq); REAL delta_euler = delta/REAL(neuler);

  REAL lambda = rho[7]; REAL mu_p = rho[8]; REAL sigma_p = rho[9]; REAL lambda_v = rho[10]; REAL mu_v = rho[11];
  realmat pars_price(4,1,0.0); pars_price[1] = lambda; pars_price[2] = mu_p; pars_price[3] = sigma_p; pars_price[4] = mu_v;
  realmat pars_vol(4,1,0.0);  pars_vol[1] = lambda_v; pars_vol[2] = mu_p; pars_vol[3] = sigma_p; pars_vol[4] = mu_v;


  REAL V_1 = theta; //continuous stochastic volatility
  REAL V_2 = theta*ind_v;
  for (INTEGER t=1; t<=T; t++) {
     for (INTEGER i=1; i<=freq; i++) {
       for (INTEGER j=1; j<=neuler; j++) {
         realmat temp_v = jumps(seed, delta_euler, pars_vol);
         V_1 += kappa_1*(theta-V_1)*delta_euler+sigma*sqrt(delta_euler)*sqrt(V_1)*unsk(seed);
         V_2 += -kappa_2*V_2*delta_euler+ind_v*temp_v[2];
       }
       realmat temp = jumps(seed, delta, pars_price); 
       output((t-1)*freq+i,1) = sqrt(V_1+V_2)*sqrt(delta)*unsk(seed)+ind_p*temp[1];
     }
  }
  return output;
}



realmat vol(INT_32BIT& seed, INTEGER T, INTEGER freq, realmat& rho)
{
  realmat output(T*freq,1,0.0);
  REAL delta = 1.0/REAL(freq);
  REAL kappa_1  = rho[1];
  REAL theta    = rho[2];
  REAL sigma    = rho[3];
  REAL kappa_2 = rho[4];
  REAL ind_v = rho[6];

  REAL lambda = rho[7]; REAL mu_p = rho[8]; REAL sigma_p = rho[9]; REAL lambda_v = rho[10]; REAL mu_v = rho[11];
  realmat pars_price(4,1,0.0); pars_price[1] = lambda; pars_price[2] = mu_p; pars_price[3] = sigma_p; pars_price[4] = mu_v;
  realmat pars_vol(4,1,0.0);  pars_vol[1] = lambda_v; pars_vol[2] = mu_p; pars_vol[3] = sigma_p; pars_vol[4] = mu_v;


  REAL V_1 = theta; //continuous stochastic volatility
  REAL V_2 = theta*ind_v;
  for (INTEGER t=1; t<=T; t++) {
    for (INTEGER j=1; j<=freq; j++) {
       output[(t-1)*freq+j] = V_1+V_2;
       realmat temp = jumps(seed, delta, pars_price); realmat temp_v = jumps(seed, delta, pars_vol);
       V_1 += kappa_1*(theta-V_1)*delta+sigma*sqrt(delta)*sqrt(V_1)*unsk(seed);
       V_2 += -kappa_2*V_2*delta+ind_v*temp_v[2];
    }
  }
  return output;
}


